package daemons

import (
	"fmt"
	"oc-schedulerd/conf"
	"time"

	oclib "cloud.o-forge.io/core/oc-lib"
	workflow_execution "cloud.o-forge.io/core/oc-lib/models/workflow_execution"
)

var Executions = ScheduledExecution{Execs: map[string]workflow_execution.WorkflowExecution{}}

type ExecutionManager struct{}

// Loop every second on the Execution's list and move the Execution that must start to a new list
// that will be looped over to start them
func (em *ExecutionManager) RetrieveNextExecutions() {
	logger := oclib.GetLogger()
	for {
		fmt.Println("Checking for executions", len(Executions.Execs))
		Executions.Mu.Lock()
		if len(Executions.Execs) > 0 {
			executions := Executions.Execs
			for execId, exec := range executions {
				if exec.ExecDate.Before(time.Now().UTC()) {
					logger.Info().Msg("Will execute " + execId + " soon")
					go em.executeExecution(&exec)
					delete(executions,execId)
				}
			}
		}
		Executions.Mu.Unlock()
		time.Sleep(time.Second)
	}
}

func (em *ExecutionManager) executeExecution(execution *workflow_execution.WorkflowExecution) {
	// start execution
	// create the yaml that describes the pod : filename, path/url to Loki
	var executor Executor
	// exec_method := os.Getenv("MONITOR_METHOD")
	logger := oclib.GetLogger()

	duration := 0
	if execution.EndDate != nil {
		duration = int(execution.EndDate.Sub(execution.ExecDate).Seconds())
	}

	if conf.GetConfig().Mode == "local" {
		executor = NewLocalMonitor(execution.UUID, execution.CreatorID, duration)
	}

	if conf.GetConfig().Mode == "container" {
		executor = NewContainerMonitor(execution.UUID, execution.CreatorID, duration)
	}

	if executor == nil {
		logger.Fatal().Msg("Could not create executor")
	}

	args := executor.PrepareMonitorExec()
	executor.LaunchMonitor(args,logger)

	// if exec_method == "k8s" {
	// 	logger.Error().Msg("TODO : executing oc-monitor in a k8s")
	// } else {
	// 	logger.Debug().Msg("Executing oc-monitor localy")
	// 	duration := 0
	// 	if Execution.EndDate != nil {
	// 		duration = int(Execution.EndDate.Sub(Execution.ExecDate).Seconds())
	// 	}
	// 	monitor := LocalMonitor{
	// 		Logger:      	logger,
	// 		Duration:    	duration,
	// 		ExecutionID: 	Execution.ExecutionsID,
	// 		PeerID:     	Execution.CreatorID,
	// 		LokiUrl:		conf.GetConfig().LokiUrl,
			
	// 	}
	// 	monitor.LaunchLocalMonitor()
	// }
}
